@@ -31,8 +31,9 @@ void Read_RC_File( void )
 
 	/* To be used in the new config section */
 	void *iter;
-	char *iterKey = NULL;
-	char *iterData = NULL;
+	gchar *iterKey = NULL;
+	gchar *iterData = NULL;
+	gchar *tmp;
 
 	CONTACT_PTR contactdata;
 
@@ -66,17 +67,17 @@ void Read_RC_File( void )
 		if( strcmp( passwd, "" ) )
 			gnome_config_clean_key( "User/Password" );
 	}
-	
+
 	our_info->nick = gnome_config_get_string( "User/Nickname=User" );
 
-	preset_status = atoi( gnome_config_get_string( "User/Status=0" ) );
+	preset_status = gnome_config_get_int( "User/Status=0" );
 
 	g_free( Away_Message );
 	Away_Message = gnome_config_get_string( "User/AwayMessage=User is currently away.  Please leave a message and have a great day!" );
 
 	g_free( our_info->first );
 	our_info->first = gnome_config_get_string( "User/First" );
-	
+
 	g_free( our_info->last );
 	our_info->last = gnome_config_get_string( "User/Last" );
 
@@ -145,7 +146,8 @@ void Read_RC_File( void )
 	max_tcp_port = gnome_config_get_int( "Server/MaxTCPPort=4100" );
 
 	/* Window info */
-	sscanf( gnome_config_get_string( "Window/Size=160x330" ), "%dx%d", &WindowWidth, &WindowHeight);
+	sscanf( (tmp=gnome_config_get_string( "Window/Size=160x330" )), "%dx%d", &WindowWidth, &WindowHeight);
+	g_free(tmp);
 
 	WindowWidth = (WindowWidth > 500) ? 500 : WindowWidth;
 	WindowHeight = (WindowHeight > 1200) ? 1200 : WindowHeight;
@@ -159,14 +161,51 @@ void Read_RC_File( void )
 
 	/* Toggles */
 	toggles->sound = gnome_config_get_bool( "Toggles/Sound=true" );
+	toggles->sound_onlyonline = gnome_config_get_bool( "Toggles/Sound_OnlyOnline=false" );
 	toggles->beep = gnome_config_get_bool( "Toggles/Beep=false" );
 	toggles->packet_udp = gnome_config_get_bool( "Toggles/PacketDumpUDP=false" );
 	toggles->packet_tcp = gnome_config_get_bool( "Toggles/PacketDumpTCP=false" );
 	toggles->force = gnome_config_get_bool( "Toggles/Force=false" );
 	toggles->autohide = gnome_config_get_bool( "Toggles/AutoHide=false" );
+	toggles->connect_startup = gnome_config_get_bool( "Toggles/ConnectStartup=true" );
 	toggles->webpresence = gnome_config_get_bool( "Toggles/WebPresence=false" );
 	toggles->show_window = gnome_config_get_bool( "Toggles/ShowWindow=false" );
+	toggles->russian = gnome_config_get_bool( "Toggles/Russian=false" );
+	toggles->check_spell = gnome_config_get_bool( "Toggles/Spelling=false" );
+	toggles->kanji = gnome_config_get_bool( "Toggles/Kanji=false" );
 	toggles->full_reply = gnome_config_get_bool( "Toggles/FullReply=true" );
+	toggles->auto_popup = gnome_config_get_bool( "Toggles/AutoPopup=false" );
+	toggles->no_new_users = gnome_config_get_bool( "Toggles/NoNewUsers=false" );
+	toggles->hidetooltip = gnome_config_get_bool( "Toggles/HideToolTip=false" );
+	toggles->status_notify = gnome_config_get_bool ("Toggles/StatusNotify=false" );
+	toggles->sort_contacts = gnome_config_get_bool( "Toggles/Sort_Contacts=true" );
+	toggles->program_on_receive_message =
+		gnome_config_get_bool("Toggles/ProgramOnReceiveMessage=false");
+	toggles->program_on_chat_request =
+		gnome_config_get_bool( "Toggles/ProgramOnChatRequest=false" );
+	toggles->program_on_receive_url =
+		gnome_config_get_bool( "Toggles/ProgramOnReceiveURL=false" );
+	toggles->program_on_file_request =
+		gnome_config_get_bool( "Toggles/ProgramOnFileRequest=false" );
+	toggles->program_on_user_online =
+		gnome_config_get_bool( "Toggles/ProgramOnUserOnline=false" );
+	toggles->program_on_user_offline =
+		gnome_config_get_bool( "Toggles/ProgramOnUserOffline=false" );
+	toggles->program_on_auth =
+		gnome_config_get_bool( "Toggles/ProgramOnAuth=false" );
+	toggles->program_on_auth_request =
+		gnome_config_get_bool( "Toggles/ProgramOnAuthReq=false" );
+	toggles->program_on_list_add =
+		gnome_config_get_bool( "Toggles/ProgramOnListAdd=false" );
+	toggles->program_on_contact_list =
+		gnome_config_get_bool( "Toggles/ProgramOnContactList=false" );
+	toggles->auto_away = gnome_config_get_int( "Toggles/AutoAway=10" );
+	toggles->auto_na = gnome_config_get_int( "Toggles/AutoNA=20" );
+	toggles->use_autoaway = gnome_config_get_bool("Toggles/UseAutoAway=true");
+	toggles->auto_accept_xfers = gnome_config_get_bool("Toggles/AutoAcceptXfers=false");
+	toggles->auto_accept_path = gnome_config_get_string("Toggles/AutoAcceptPath=~/.icq/Received");
+
+        toggles->theme_name = gnome_config_get_string( "Toggles/ThemeName=Default" );
 
 	/* Auto Hide */
 	autohide_time = gnome_config_get_int( "AutoHide/AutoHideTime=60" );
@@ -178,22 +217,52 @@ void Read_RC_File( void )
 	if( ChatFontString == NULL || !strcmp( ChatFontString, "" ) )
 		ChatFontString = g_strdup( "-adobe-courier-medium-r-normal-*-*-140-*-*-m-*-iso8859-1" );
 
+	/* External Programs */
+	programs->receive_message =
+		gnome_config_get_string( "Programs/OnRecieveMessage" );
+	programs->chat_request =
+		gnome_config_get_string( "Programs/OnChatRequest" );
+	programs->receive_url =
+		gnome_config_get_string( "Programs/OnReceiveURL" );
+	programs->file_request =
+		gnome_config_get_string( "Programs/OnFileRequest" );
+	programs->user_online =
+		gnome_config_get_string( "Programs/OnUserOnline" );
+	programs->user_offline =
+		gnome_config_get_string( "Programs/OnUserOffline" );
+	programs->auth =
+		gnome_config_get_string( "Programs/OnAuth" );
+	programs->auth_request =
+		gnome_config_get_string( "Programs/OnAuthRequest" );
+	programs->list_add =
+		gnome_config_get_string( "Programs/OnListAdd" );
+	programs->contact_list =
+		gnome_config_get_string( "Programs/OnContactList" );
+
 	/* Color section */
-	str2col(&color_online, gnome_config_get_string( "Color/Online=#000000009a69009a" ) );
+	str2col(&color_online, (tmp=gnome_config_get_string( "Color/Online=#000000009a69009a" )) );
+	g_free(tmp);
 	color_online_set = TRUE;
-	str2col(&color_offline, gnome_config_get_string( "Color/Offline=#75d675d675d67575" ) );
+	str2col(&color_offline, (tmp=gnome_config_get_string( "Color/Offline=#75d675d675d67575" )) );
+	g_free(tmp);
 	color_offline_set = TRUE;
-	str2col(&color_away, gnome_config_get_string( "Color/Away=#0000820700008200" ) );
+	str2col(&color_away, (tmp=gnome_config_get_string( "Color/Away=#0000820700008200" )) );
+	g_free(tmp);
 	color_away_set = TRUE;
-	str2col(&color_na, gnome_config_get_string( "Color/NotAvailable=#0000820700008200" ) );
+	str2col(&color_na, (tmp=gnome_config_get_string( "Color/NotAvailable=#0000820700008200" )) );
+	g_free(tmp);
 	color_na_set = TRUE;
-	str2col(&color_ffc, gnome_config_get_string( "Color/FreeForChat=#0000000075300075" ) );
+	str2col(&color_ffc, (tmp=gnome_config_get_string( "Color/FreeForChat=#0000000075300075" )) );
+	g_free(tmp);
 	color_ffc_set = TRUE;
-	str2col(&color_occ, gnome_config_get_string( "Color/Occupied=#0000000075300075" ) );
+	str2col(&color_occ, (tmp=gnome_config_get_string( "Color/Occupied=#0000000075300075" )) );
+	g_free(tmp);
 	color_occ_set = TRUE;
-	str2col(&color_dnd, gnome_config_get_string( "Color/DoNotDisturb=#0000000075300075" ) );
+	str2col(&color_dnd, (tmp=gnome_config_get_string( "Color/DoNotDisturb=#0000000075300075" )) );
+	g_free(tmp);
 	color_dnd_set = TRUE;
-	str2col(&color_inv, gnome_config_get_string( "Color/Invisible=#9a692ed2810c2e81" ) );
+	str2col(&color_inv, (tmp=gnome_config_get_string( "Color/Invisible=#9a692ed2810c2e81" )) );
+	g_free(tmp);
 	color_inv_set = TRUE;
 
 	/* New Contacts section */
@@ -202,7 +271,11 @@ void Read_RC_File( void )
 		iter = gnome_config_init_iterator( "NewContacts" );
 		while( 1 )
 		{
+			g_free( iterKey );
+			g_free( iterData );
+
 			iter = gnome_config_iterator_next( iter, &iterKey, &iterData );
+
 			if( iter == NULL )
 				break;
 			if( atoi( iterKey ) == 0 )
@@ -213,6 +286,11 @@ void Read_RC_File( void )
 			else
 				contactdata->online_notify = FALSE;
 
+			if( strstr( iterData, "Ssrv" ) ) /* Force through server */
+				contactdata->force_override = TRUE;
+			else
+				contactdata->force_override = FALSE;
+
 			if( strstr( iterData, "Sign" ) ) /* Ignore List */
 				contactdata->ignore_list = TRUE;
 			else
@@ -227,7 +305,7 @@ void Read_RC_File( void )
 				contactdata->invis_list = FALSE;
 				contactdata->vis_list = FALSE;
 			}
-		
+
 			contactdata->uin = atoi( iterKey );
 
 			if( iterData[ 0 ] == ',' )
@@ -242,42 +320,16 @@ void Read_RC_File( void )
 					         sizeof( contactdata->nick ) );
 			}
 
-			contactdata->oneline_in = NULL;
-			contactdata->oneline_out = NULL;
 			contactdata->sok = 0;
-			contactdata->chat_away = 0;
-			contactdata->chat_window = NULL;
-			contactdata->chat_sok = 0;
-			contactdata->chat_seq = 0;
-			contactdata->chat_active = FALSE;
-			contactdata->chat_active2 = FALSE;
-			contactdata->chat_port = 0;
-			contactdata->chat_history = NULL;
-			contactdata->chat_bg_red =
-			contactdata->chat_bg_green =
-			contactdata->chat_bg_blue = 0;
-			contactdata->chat_fg_red =
-			contactdata->chat_fg_green =
-			contactdata->chat_fg_blue = 255;
-			contactdata->tcp_gdk_input = 0;
-			contactdata->chat_gdk_input = 0;
-			contactdata->pdata = NULL;
-			contactdata->file_name = NULL;
-			contactdata->file_short_name = NULL;
-			contactdata->file_fd = 0;
-			contactdata->file_offset = 0;
-			contactdata->file_sending = FALSE;
+			contactdata->chat_requests = NULL;
 			contactdata->stored_messages = NULL;
 			contactdata->status = STATUS_OFFLINE;
-			contactdata->true_status = STATUS_OFFLINE;
 			contactdata->last_time = -1L;
 			contactdata->current_ip = -1L;
-			contactdata->gdk_input_tag = 0;
-			contactdata->file_info = NULL;
 			contactdata->show_again = TRUE;
-			contactdata->chat_reason = NULL;
 			contactdata->info = g_new0( USER_INFO_STRUCT, 1 );
 			contactdata->inlist = TRUE;
+			contactdata->tcp_seq = -1;
 
 			Contacts = g_slist_insert_sorted( Contacts, contactdata, (GCompareFunc)sort_compare );
 		}
@@ -313,43 +365,17 @@ void Read_RC_File( void )
 				contactdata->vis_list = FALSE;
 				contactdata->uin = atoi( temp_uin );
 			}
-			contactdata->oneline_in = NULL;
-			contactdata->oneline_out = NULL;
 			contactdata->sok = 0;
-			contactdata->chat_away = 0;
-			contactdata->chat_window = NULL;
-			contactdata->chat_sok = 0;
-			contactdata->chat_seq = 0;
-			contactdata->chat_active = FALSE;
-			contactdata->chat_active2 = FALSE;
-			contactdata->chat_port = 0;
-			contactdata->chat_history = NULL;
-			contactdata->chat_bg_red =
-			contactdata->chat_bg_green =
-			contactdata->chat_bg_blue = 0;
-			contactdata->chat_fg_red =
-			contactdata->chat_fg_green =
-			contactdata->chat_fg_blue = 255;
-			contactdata->tcp_gdk_input = 0;
-			contactdata->chat_gdk_input = 0;
-			contactdata->pdata = NULL;
-			contactdata->file_name = NULL;
-			contactdata->file_short_name = NULL;
-			contactdata->file_fd = 0;
-			contactdata->file_offset = 0;
-			contactdata->file_sending = FALSE;
-			contactdata->message = NULL;
+			contactdata->chat_requests = NULL;
 			contactdata->messages = 0;
 			contactdata->status = STATUS_OFFLINE;
-			contactdata->true_status = STATUS_OFFLINE;
 			contactdata->last_time = -1L;
 			contactdata->current_ip = -1L;
-			contactdata->gdk_input_tag = 0;
-			contactdata->file_info = NULL;
 			contactdata->show_again = TRUE;
-			contactdata->chat_reason = NULL;
 			contactdata->info = g_new0( USER_INFO_STRUCT, 1 );
 
+			contactdata->tcp_seq = -1;
+
 			if( temp_nick )
 			{
 				strncpy( contactdata->nick, temp_nick, sizeof(contactdata->nick)-1);
@@ -358,16 +384,23 @@ void Read_RC_File( void )
 			else
 				contactdata->nick[ 0 ] = 0;
 
-			Contacts = g_slist_append( Contacts, contactdata );
+			Contacts = g_slist_prepend( Contacts, contactdata );
 		}
 
+		Contacts = g_slist_reverse( Contacts );
+
 		g_free( true_value );
 	}
+	/* Here we get all of the away messages */
+	get_config_file_away_msgs();
+	/* End get all of the away messages */
+
 
 	gnome_config_pop_prefix();
 
 	have_read_config = TRUE;
 
+
 	return;
 }
 
@@ -465,15 +498,51 @@ void Save_RC( void )
 
 	/* Toggles */
 	gnome_config_set_bool( "Toggles/Sound", toggles->sound );
+	gnome_config_set_bool( "Toggles/Sound_OnlyOnline", toggles->sound_onlyonline );
 	gnome_config_set_bool( "Toggles/Beep", toggles->beep );
 	gnome_config_set_bool( "Toggles/PacketDumpUDP", toggles->packet_udp );
 	gnome_config_set_bool( "Toggles/PacketDumpTCP", toggles->packet_tcp );
 	gnome_config_set_bool( "Toggles/Force", toggles->force );
 	gnome_config_set_bool( "Toggles/AutoHide", toggles->autohide );
+	gnome_config_set_bool( "Toggles/ConnectStartup", toggles->connect_startup );
+	gnome_config_set_bool( "Toggles/HideToolTip", toggles->hidetooltip );
+	gnome_config_set_bool( "Toggles/Sort_Contacts", toggles->sort_contacts );
 	gnome_config_set_bool( "Toggles/WebPresence", toggles->webpresence );
 	gnome_config_clean_key( "Toggles/Headers" );
 	gnome_config_set_bool( "Toggles/ShowWindow", toggles->show_window );
+	gnome_config_set_bool( "Toggles/Spelling", toggles->check_spell );
+	gnome_config_set_bool( "Toggles/Russian", toggles->russian );
+	gnome_config_set_bool( "Toggles/Kanji", toggles->kanji );
 	gnome_config_set_bool( "Toggles/FullReply", toggles->full_reply );
+	gnome_config_set_bool( "Toggles/AutoPopup", toggles->auto_popup );
+	gnome_config_set_bool( "Toggles/StatusNotify", toggles->status_notify );
+	gnome_config_set_bool( "Toggles/NoNewUsers", toggles->no_new_users );
+	gnome_config_set_bool( "Toggles/ProgramOnReceiveMessage",
+			       toggles->program_on_receive_message );
+	gnome_config_set_bool( "Toggles/ProgramOnChatRequest",
+			       toggles->program_on_chat_request );
+	gnome_config_set_bool( "Toggles/ProgramOnReceiveURL",
+			       toggles->program_on_receive_url );
+	gnome_config_set_bool( "Toggles/ProgramOnFileRequest",
+			       toggles->program_on_file_request );
+	gnome_config_set_bool( "Toggles/ProgramOnUserOnline",
+			       toggles->program_on_user_online );
+	gnome_config_set_bool( "Toggles/ProgramOnUserOffline",
+			       toggles->program_on_user_offline );
+	gnome_config_set_bool( "Toggles/ProgramOnAuth",
+			       toggles->program_on_auth );
+	gnome_config_set_bool( "Toggles/ProgramOnAuthReq",
+			       toggles->program_on_auth_request );
+	gnome_config_set_bool( "Toggles/ProgramOnListAdd",
+			       toggles->program_on_list_add );
+	gnome_config_set_bool( "Toggles/ProgramOnContactList",
+			       toggles->program_on_contact_list );
+	gnome_config_set_int( "Toggles/AutoAway", toggles->auto_away);
+	gnome_config_set_int( "Toggles/AutoNA", toggles->auto_na);
+	gnome_config_set_bool( "Toggles/UseAutoAway", toggles->use_autoaway);
+	gnome_config_set_bool( "Toggles/AutoAcceptXfers", toggles->auto_accept_xfers);
+	gnome_config_set_string( "Toggles/AutoAcceptPath", toggles->auto_accept_path);
+        gnome_config_set_string( "Toggles/ThemeName", toggles->theme_name );
 
 	/* AutoHide */
 	gnome_config_set_int( "AutoHide/AutoHideTime", autohide_time );
@@ -538,6 +607,28 @@ void Save_RC( void )
 	/* Chat font */
 	gnome_config_set_string( "Color/ChatFont", ChatFontString );
 
+	/* External Programs */
+	gnome_config_set_string( "Programs/OnRecieveMessage",
+				 programs->receive_message );
+	gnome_config_set_string( "Programs/OnChatRequest",
+				 programs->chat_request );
+	gnome_config_set_string( "Programs/OnReceiveURL",
+				 programs->receive_url );
+	gnome_config_set_string( "Programs/OnFileRequest",
+				 programs->file_request );
+	gnome_config_set_string( "Programs/OnUserOnline",
+				 programs->user_online );
+	gnome_config_set_string( "Programs/OnUserOffline",
+				 programs->user_offline );
+	gnome_config_set_string( "Programs/OnAuth",
+				 programs->auth );
+	gnome_config_set_string( "Programs/OnAuthRequest",
+				 programs->auth_request );
+	gnome_config_set_string( "Programs/OnListAdd",
+				 programs->list_add );
+	gnome_config_set_string( "Programs/OnContactList",
+				 programs->contact_list );
+
 	/* New Contact Section */
 	gnome_config_clean_section( "NewContacts" );
 
@@ -545,39 +636,51 @@ void Save_RC( void )
 
 	while( contact != NULL )
 	{
+		if( kontakt->inlist == FALSE )
+		{
+			contact = contact->next;
+			continue;
+		}
 		contactKey = g_strdup_printf( "NewContacts/%u",
-					      ( (CONTACT_PTR)( (CONTACT_PTR)contact->data ) )->uin );
+					      kontakt->uin );
 
-		contactStr = g_strdup_printf( "%s%s%s%s%s%s",
+		contactStr = g_strdup_printf( "%s%s%s%s%s%s%s",
 
 	/* Start with the nickname */
-					      ((CONTACT_PTR)((CONTACT_PTR)contact->data))->nick,
+					      (kontakt)->nick,
 
 	/* If we append ANYTHING, we need a comma here */
-					      ( ((CONTACT_PTR)contact->data)->invis_list ||
-		                 ((CONTACT_PTR)contact->data)->vis_list ||
-		                 ((CONTACT_PTR)contact->data)->ignore_list ||
-		                 ((CONTACT_PTR)contact->data)->online_notify ? "," : "" ),
+					      ( kontakt->invis_list ||
+		                 kontakt->vis_list ||
+		                 kontakt->ignore_list ||
+		                 kontakt->force_override ||
+		                 kontakt->online_notify ? "," : "" ),
+
+	/* If we're forcing their messages thru server, we append Ssrv */
+	                  ( kontakt->force_override ? "Ssrv" : "" ),
 
 	/* If they're on the invisible list, we append an Sinv */
-		               ( ((CONTACT_PTR)contact->data)->invis_list ? "Sinv" : "" ),
+		               ( kontakt->invis_list ? "Sinv" : "" ),
 
 	/* If they're on the visible list, we append Svis here */
-		               ( ((CONTACT_PTR)contact->data)->vis_list ? "Svis" : "" ),
+		               ( kontakt->vis_list ? "Svis" : "" ),
 
 	/* If they're on the notify list, we append Snot here */
-		               ( ((CONTACT_PTR)contact->data)->online_notify ? "Snot" : "" ),
-		
-	/* If they're on the ignore list, we append Sign here */
-		               ( ((CONTACT_PTR)contact->data)->ignore_list ? "Sign" : "" ) );
+		               ( kontakt->online_notify ? "Snot" : "" ),
 
-		if( ((CONTACT_PTR)contact->data)->inlist == TRUE )
-			gnome_config_set_string( contactKey, contactStr );
+	/* If they're on the ignore list, we append Sign here */
+		               ( kontakt->ignore_list ? "Sign" : "" ) );
+
+		gnome_config_set_string( contactKey, contactStr );
 		g_free( contactKey );
 		g_free( contactStr );
 		contact = contact->next;
 	}
 
+	/* Write the away messages to the config file */
+	set_config_file_away_msgs();
+	/* End of writing the away messages to the config file */
+
 	gnome_config_set_int( "Existence/exists", 1 );
 
 	gnome_config_sync();
@@ -601,5 +704,6 @@ void str2col(GdkColor *color, const gchar *str) {
 
 int sort_compare( CONTACT_PTR c1, CONTACT_PTR c2 )
 {
-	return strcmp( c1->nick, c2->nick );
+
+	return strcoll( c1->nick, c2->nick );
 }
