@@ -503,9 +503,11 @@ int main( int argc, char *argv[] )
 
 #ifdef GNOME_ICON
 	{
-		gchar *icon_path = make_file_path_theme( "gnomeicu-online.png", "Default" );
-		gnome_window_icon_set_default_from_file( icon_path );
+		gchar *icon_path = gnome_unconditional_datadir_file ("pixmaps/");
+		gchar *master_icon = g_strconcat (icon_path, "gnome-gnomeicu.xpm", NULL);
+		gnome_window_icon_set_default_from_file( master_icon );
 		g_free( icon_path );
+		g_free (master_icon);
 	}
 #endif
 
@@ -1037,3 +1039,13 @@ void change_userwin( GtkWidget *widget, GtkNotebookPage *page, int num,
 	else if( num == 2 )
 		MainData->lb_userwin = MainData->lb_userwin_notinlist;
 }
+
+void go_url_home (void)
+{
+	gnome_url_show ("http://gnomeicu.sourceforge.net/index.php");
+}
+
+void go_url_updates (void)
+{
+	gnome_url_show ("http://gnomeicu.sourceforge.net/updates.php");
+}
