@@ -89,5 +89,7 @@ void ready_set( void );
 void create_animator( GtkWidget *hbox_parent );
 void toggle_log_window( GtkWidget *widget, gpointer data );
 void init( void );
+void go_url_home (void);
+void go_url_updates (void);
 
 #endif /* __GNOMEICU_H__ */
