@@ -46,23 +46,12 @@ void log_window_add( const gchar *statement, UIN_T uin )
 	         my_tm->tm_sec, halves[ half ] );
 	sprintf( pdate, "%02d/%02d/%d", my_tm->tm_mon + 1, my_tm->tm_mday,
 	         my_tm->tm_year + 1900 );
-
-	contact = Contacts;
-
-	while( contact != NULL )
-	{
-		if( uin == ((CONTACT_PTR)contact->data)->uin )
-		{
-			sprintf( who, "%s", ((CONTACT_PTR)contact->data)->nick );
-			break;
-		}
-		contact = contact->next;
-	}
 	
+	contact = Find_User( uin );
 	if( contact == NULL )
-	{
 		sprintf( who, "%d", uin );
-	}
+	else
+		sprintf( who, "%s", kontakt->nick );
 
 	gtk_clist_insert( GTK_CLIST( log_list ), 0, strings );
 }
