@@ -120,12 +120,17 @@ void Check_Queue( WORD seq )
 void msg_queue_clear( void )
 {
    int i;
+   struct msg *message;
 	
    g_assert( queue != NULL );
    g_assert( 0 <= queue->entries );
    
    for (i = 0; i < queue->entries; i++)
    {
-       msg_queue_pop();
+       message = msg_queue_pop();
+       if (message) {
+           g_free(message->body);
+           g_free(message);
+       }
    }
 }
