@@ -980,8 +980,7 @@ GSList *Do_File( char *data, UIN_T uin, DWORD seq, gchar *file_name, int file_si
 			applet_update( Current_Status, NULL );
 		Show_Quick_Status();
 
-		/* FIXME: CHAT?? This should be FILE... */
-		gnomeicu_event( EV_CHATREQ,uin );
+		gnomeicu_event( EV_FILEREQ,uin );
 	}
 
 	return contact;
