@@ -3,10 +3,16 @@
 GdkImlibImage *create_image( const gchar *filename);
 void can_continue( GnomeDruidPage *page, GnomeDruid *druid );
 gint cant_continue( GnomeDruidPage *page, GnomeDruid *druid );
+gint cant_continue_back( GnomeDruidPage *page, GnomeDruid *druid );
 void can_finish( GtkWidget *widget, gint row, gint col, GdkEventButton *event,
                  gpointer *data );
-gint prepare_finish_page( GnomeDruidPage *page, GnomeDruid *druid );
-void finish_adding_user( GnomeDruidPage *page, GnomeDruid *druid );
+gint prepare_finish_page( GnomeDruidPage *page, GnomeDruid *druid, GtkWidget *window );
+/*void finish_adding_user( GnomeDruidPage *page, GnomeDruid *druid );*/
+void sort_found_list( GtkCList *clist, gint column, gpointer data );
+gint found_list_compare_func( GtkCList *clist, gconstpointer ptrt,
+                              gconstpointer ptr2 );
+void search_changed( GtkWidget *widget, gpointer data );
+void clear_i_window2( GtkWidget *widget, gpointer data );
 
 GtkWidget*
 create_AddContact (void)
@@ -14,11 +20,6 @@ create_AddContact (void)
 	GtkWidget *AddContact;
 	GtkWidget *viewport1;
 	GtkWidget *druid1;
-	GtkWidget *druidpagestart1;
-	GdkColor druidpagestart1_bg_color = { 0, 257, 257, 257 };
-	GdkColor druidpagestart1_textbox_color = { 0, 65535, 65535, 65535 };
-	GdkColor druidpagestart1_logo_bg_color = { 0, 257, 257, 257 };
-	GdkColor druidpagestart1_title_color = { 0, 65535, 65535, 65535 };
 	GtkWidget *druidpagestandard1;
 	GdkColor druidpagestandard1_bg_color = { 0, 257, 257, 257 };
 	GdkColor druidpagestandard1_logo_bg_color = { 0, 257, 257, 257 };
@@ -59,12 +60,7 @@ create_AddContact (void)
 	GtkWidget *label76;
 	GtkWidget *label77;
 	GtkWidget *label78;
-	GtkWidget *label79;
-	GtkWidget *druidpagefinish1;
-	GdkColor druidpagefinish1_bg_color = { 0, 0, 0, 0 };
-	GdkColor druidpagefinish1_textbox_color = { 0, 65535, 65535, 65535 };
-	GdkColor druidpagefinish1_logo_bg_color = { 0, 0, 0, 0 };
-	GdkColor druidpagefinish1_title_color = { 0, 65535, 65535, 65535 };
+	GtkWidget *label79;                
 
 	AddContact = gtk_window_new (GTK_WINDOW_DIALOG);
 	gtk_widget_set_name (AddContact, "AddContact");
@@ -91,23 +87,6 @@ create_AddContact (void)
 	gtk_widget_show (druid1);
 	gtk_container_add (GTK_CONTAINER (viewport1), druid1);
 
-	druidpagestart1 = gnome_druid_page_start_new ();
-	gtk_widget_set_name (druidpagestart1, "druidpagestart1");
-	gtk_widget_ref (druidpagestart1);
-	gtk_object_set_data_full (GTK_OBJECT (AddContact), "druidpagestart1", druidpagestart1,
-	                          (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (druidpagestart1);
-	gnome_druid_append_page (GNOME_DRUID (druid1), GNOME_DRUID_PAGE (druidpagestart1));
-	gnome_druid_set_page (GNOME_DRUID (druid1), GNOME_DRUID_PAGE (druidpagestart1));
-	gnome_druid_page_start_set_bg_color (GNOME_DRUID_PAGE_START (druidpagestart1), &druidpagestart1_bg_color);
-	gnome_druid_page_start_set_textbox_color (GNOME_DRUID_PAGE_START (druidpagestart1), &druidpagestart1_textbox_color);
-	gnome_druid_page_start_set_logo_bg_color (GNOME_DRUID_PAGE_START (druidpagestart1), &druidpagestart1_logo_bg_color);
-	gnome_druid_page_start_set_title_color (GNOME_DRUID_PAGE_START (druidpagestart1), &druidpagestart1_title_color);
-	gnome_druid_page_start_set_title (GNOME_DRUID_PAGE_START (druidpagestart1), _("Add A Contact"));
-	gnome_druid_page_start_set_text (GNOME_DRUID_PAGE_START (druidpagestart1), _("This will guide you through the process\nof adding a Contact to your Contact List"));
-	gnome_druid_page_start_set_watermark (GNOME_DRUID_PAGE_START (druidpagestart1),
-	                                      create_image ("gnomeicu/watermark.png"));
-
 	druidpagestandard1 = gnome_druid_page_standard_new_with_vals ("", NULL);
 	gtk_widget_set_name (druidpagestandard1, "druidpagestandard1");
 	gtk_widget_ref (druidpagestandard1);
@@ -169,6 +148,7 @@ create_AddContact (void)
 	gtk_object_set_data_full (GTK_OBJECT (AddContact), "search_uin", search_uin,
 	                          (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (search_uin);
+    gtk_widget_grab_focus( search_uin );
 
 	gtk_signal_connect( GTK_OBJECT( search_uin ), "changed",
 	                    GTK_SIGNAL_FUNC( can_continue ),
@@ -401,6 +381,8 @@ create_AddContact (void)
 	gtk_widget_ref (found_list);
 	gtk_object_set_data_full (GTK_OBJECT (AddContact), "found_list", found_list,
 	                          (GtkDestroyNotify) gtk_widget_unref);
+	gtk_clist_set_selection_mode( GTK_CLIST( found_list ), GTK_SELECTION_BROWSE );
+	gtk_clist_set_compare_func( GTK_CLIST( found_list ), found_list_compare_func );
 	gtk_widget_show (found_list);
 	gtk_container_add (GTK_CONTAINER (scrolledwindow9), found_list);
 	gtk_clist_set_row_height( GTK_CLIST( found_list ), 20 );
@@ -411,6 +393,9 @@ create_AddContact (void)
 	gtk_clist_set_column_width (GTK_CLIST (found_list), 4, 110);
 	gtk_clist_column_titles_show (GTK_CLIST (found_list));
 
+	gtk_signal_connect (GTK_OBJECT (found_list), "destroy",
+	                    GTK_SIGNAL_FUNC (clear_i_window2), NULL);
+
 	gtk_object_set_data( GTK_OBJECT( found_list ), "druid", druid1 );
 
 	label75 = gtk_label_new (_("UIN"));
@@ -461,28 +446,11 @@ create_AddContact (void)
 	gtk_clist_set_column_widget (GTK_CLIST (found_list), 4, label79);
 	gtk_label_set_justify (GTK_LABEL (label79), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC (label79), 0, 0.5);
-
-	druidpagefinish1 = gnome_druid_page_finish_new ();
-	gtk_widget_set_name (druidpagefinish1, "druidpagefinish1");
-	gtk_widget_ref (druidpagefinish1);
-	gtk_object_set_data_full (GTK_OBJECT (AddContact), "druidpagefinish1", druidpagefinish1,
-	                          (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (druidpagefinish1);
-	gnome_druid_append_page (GNOME_DRUID (druid1), GNOME_DRUID_PAGE (druidpagefinish1));
-	gnome_druid_page_finish_set_bg_color (GNOME_DRUID_PAGE_FINISH (druidpagefinish1), &druidpagefinish1_bg_color);
-	gnome_druid_page_finish_set_textbox_color (GNOME_DRUID_PAGE_FINISH (druidpagefinish1), &druidpagefinish1_textbox_color);
-	gnome_druid_page_finish_set_logo_bg_color (GNOME_DRUID_PAGE_FINISH (druidpagefinish1), &druidpagefinish1_logo_bg_color);
-	gnome_druid_page_finish_set_title_color (GNOME_DRUID_PAGE_FINISH (druidpagefinish1), &druidpagefinish1_title_color);
-	gnome_druid_page_finish_set_title (GNOME_DRUID_PAGE_FINISH (druidpagefinish1), _("Finished"));
-/*	gnome_druid_page_finish_set_text (GNOME_DRUID_PAGE_FINISH (druidpagefinish1), _("Click \"Finish\" to add <Nick> (<FName> <LName) to your\nContact List."));*/
-	gnome_druid_page_finish_set_watermark (GNOME_DRUID_PAGE_FINISH (druidpagefinish1),
-	                                      create_image ("gnomeicu/watermark.png"));
-
-	gtk_object_set_data( GTK_OBJECT( druidpagestart1 ), "next_page", druidpagestandard1 );
+        
 	gtk_object_set_data( GTK_OBJECT( druidpagestandard1 ), "next_page", druidpagestandard2 );
-	gtk_object_set_data( GTK_OBJECT( druidpagestandard2 ), "next_page", druidpagefinish1 );
 
 	gtk_object_set_data( GTK_OBJECT( druidpagestandard2 ), "clist", found_list );
+	gtk_object_set_data( GTK_OBJECT( druidpagestandard2 ), "w", AddContact );
 
 	gtk_object_set_data( GTK_OBJECT( druidpagestandard1 ), "search_uin", search_uin );
 	gtk_object_set_data( GTK_OBJECT( druidpagestandard1 ), "search_nick", search_nick );
@@ -491,25 +459,42 @@ create_AddContact (void)
 	gtk_object_set_data( GTK_OBJECT( druidpagestandard1 ), "search_email", search_email );
 	gtk_object_set_data( GTK_OBJECT( druidpagestandard1 ), "found_list", found_list );
 
-	gtk_signal_connect( GTK_OBJECT( druidpagestandard1 ), "next",
-	                    GTK_SIGNAL_FUNC( begin_search ), NULL );
+	gtk_object_set_data( GTK_OBJECT( druid1 ), "search_uin", search_uin );
+	gtk_object_set_data( GTK_OBJECT( druid1 ), "search_nick", search_nick );
+	gtk_object_set_data( GTK_OBJECT( druid1 ), "search_first", search_first );
+	gtk_object_set_data( GTK_OBJECT( druid1 ), "search_last", search_last );
+	gtk_object_set_data( GTK_OBJECT( druid1 ), "search_email", search_email );
+
+	gtk_signal_connect( GTK_OBJECT( search_uin ), "changed",
+	                    GTK_SIGNAL_FUNC( search_changed ),
+	                    druid1 );
+
+	gtk_signal_connect( GTK_OBJECT( search_nick ), "changed",
+	                    GTK_SIGNAL_FUNC( search_changed ),
+	                    druid1 );
+
+	gtk_signal_connect( GTK_OBJECT( search_first ), "changed",
+	                    GTK_SIGNAL_FUNC( search_changed ),
+	                    druid1 );
+
+	gtk_signal_connect( GTK_OBJECT( search_last ), "changed",
+	                    GTK_SIGNAL_FUNC( search_changed ),
+	                    druid1 );
+
+	gtk_signal_connect( GTK_OBJECT( search_email ), "changed",
+	                    GTK_SIGNAL_FUNC( search_changed ),
+	                    druid1 );
+
+	gtk_signal_connect_after( GTK_OBJECT( druidpagestandard1 ), "next",
+	                                      GTK_SIGNAL_FUNC( begin_search ),
+	                                      NULL );
 
-	gtk_signal_connect_object( GTK_OBJECT( druidpagestart1 ), "cancel",
-	                           GTK_SIGNAL_FUNC( gtk_widget_destroy ),
-	                           GTK_OBJECT( AddContact ) );
 	gtk_signal_connect_object( GTK_OBJECT( druidpagestandard1 ), "cancel",
 	                           GTK_SIGNAL_FUNC( gtk_widget_destroy ),
 	                           GTK_OBJECT( AddContact ) );
 	gtk_signal_connect_object( GTK_OBJECT( druidpagestandard2 ), "cancel",
 	                           GTK_SIGNAL_FUNC( gtk_widget_destroy ),
 	                           GTK_OBJECT( AddContact ) );
-	gtk_signal_connect_object( GTK_OBJECT( druidpagefinish1 ), "cancel",
-	                           GTK_SIGNAL_FUNC( gtk_widget_destroy ),
-	                           GTK_OBJECT( AddContact ) );
-
-	gtk_signal_connect( GTK_OBJECT( druidpagestart1 ), "next",
-	                    GTK_SIGNAL_FUNC( cant_continue ),
-	                    NULL );
 
 	gtk_signal_connect( GTK_OBJECT( druidpagestandard1 ), "next",
 	                    GTK_SIGNAL_FUNC( cant_continue ),
@@ -517,19 +502,23 @@ create_AddContact (void)
 
 	gtk_signal_connect( GTK_OBJECT( druidpagestandard2 ), "next",
 	                    GTK_SIGNAL_FUNC( prepare_finish_page ),
+	                    AddContact );
+
+	gtk_signal_connect( GTK_OBJECT( druidpagestandard2 ), "back",
+	                    GTK_SIGNAL_FUNC( cant_continue_back ),
 	                    NULL );
 
 	gtk_signal_connect( GTK_OBJECT( found_list ), "select_row",
 	                    GTK_SIGNAL_FUNC( can_finish ),
 	                    druid1 );
 
-	gtk_signal_connect( GTK_OBJECT( druidpagefinish1 ), "finish",
-	                    GTK_SIGNAL_FUNC( finish_adding_user ),
+	gtk_signal_connect( GTK_OBJECT( found_list ), "click_column",
+	                    GTK_SIGNAL_FUNC( sort_found_list ),
 	                    NULL );
 
-	gtk_signal_connect_object( GTK_OBJECT( druidpagefinish1 ), "finish",
-	                           GTK_SIGNAL_FUNC( gtk_widget_destroy ),
-	                           GTK_OBJECT( AddContact ) );
+	gtk_object_set_data( GTK_OBJECT( druidpagestandard2 ), "prev_page", druidpagestandard1 );
+
+	gnome_druid_set_buttons_sensitive( GNOME_DRUID( druid1 ), FALSE, FALSE, TRUE );
 
 	return AddContact;
 }
@@ -553,21 +542,62 @@ GdkImlibImage *create_image( const gchar *filename)
 
 void can_continue( GnomeDruidPage *page, GnomeDruid *druid )
 {
-	gnome_druid_set_buttons_sensitive( druid, TRUE, TRUE, TRUE );
+	gpointer prev_page = NULL;
+
+	prev_page = gtk_object_get_data( GTK_OBJECT( page ), "prev_page" );
+
+	if( prev_page == NULL )
+            gnome_druid_set_buttons_sensitive( druid, FALSE, TRUE, TRUE );
+	else
+            gnome_druid_set_buttons_sensitive( druid, TRUE, TRUE, TRUE );
 
 }
 
 gint cant_continue( GnomeDruidPage *page, GnomeDruid *druid )
 {
 	GnomeDruidPage *next_page;
+	gpointer prev_page;
 
 	next_page = GNOME_DRUID_PAGE( gtk_object_get_data( GTK_OBJECT( page ), "next_page" ) );
+	prev_page = gtk_object_get_data( GTK_OBJECT( page ), "prev_page" );
 	gnome_druid_set_page( druid, next_page );
+
 	gnome_druid_set_buttons_sensitive( druid, TRUE, FALSE, TRUE );
 
 	return TRUE;
 }
 
+gint cant_continue_back( GnomeDruidPage *page, GnomeDruid *druid )
+{
+	gpointer prev_page;
+	GtkEntry *e_uin;
+	GtkEntry *e_nick;
+	GtkEntry *e_first;
+	GtkEntry *e_last;
+	GtkEntry *e_email;
+
+
+	prev_page = gtk_object_get_data( GTK_OBJECT( page ), "prev_page" );
+	gnome_druid_set_page( druid, prev_page );
+
+	e_uin = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_uin" ) );
+	e_nick = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_nick" ) );
+	e_first = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_first" ) );
+	e_last = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_last" ) );
+	e_email = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_email" ) );
+
+	if( !strcmp( gtk_entry_get_text( e_uin ), "" ) &&
+	    !strcmp( gtk_entry_get_text( e_nick ), "" ) &&
+	    !strcmp( gtk_entry_get_text( e_first ), "" ) &&
+	    !strcmp( gtk_entry_get_text( e_last ), "" ) &&
+	    !strcmp( gtk_entry_get_text( e_email ), "" ) )
+		gnome_druid_set_buttons_sensitive( druid, FALSE, FALSE, TRUE );
+	else
+		gnome_druid_set_buttons_sensitive( druid, FALSE, TRUE, TRUE );
+
+	return TRUE;
+}
+
 void can_finish( GtkWidget *widget, gint row, gint col, GdkEventButton *event,
                  gpointer *data )
 {
@@ -576,14 +606,15 @@ void can_finish( GtkWidget *widget, gint row, gint col, GdkEventButton *event,
 	gnome_druid_set_buttons_sensitive( druid, TRUE, TRUE, TRUE );
 }
 
-gint prepare_finish_page( GnomeDruidPage *page, GnomeDruid *druid )
+/* TODO: use GnomeDruidPageFinish instead of another dialog */
+gint prepare_finish_page( GnomeDruidPage *page, GnomeDruid *druid, GtkWidget *window )
 {
-	GnomeDruidPage *druidpagefinish1 = GNOME_DRUID_PAGE( gtk_object_get_data( GTK_OBJECT( page ), "next_page" ) );
 	GtkCList *clist = GTK_CLIST( gtk_object_get_data( GTK_OBJECT( page ), "clist" ) );
+        GtkWidget *finish_message;
 	gint row = GPOINTER_TO_INT( clist->selection->data );
 	gchar *str;
 	UIN_T uin;
-	gchar *nick, *fname, *lname;
+	gchar *nick = NULL, *fname, *lname, *msg = NULL;
 
 	gtk_clist_get_text( clist, row, 0, &str );
 	uin = atoi( str );
@@ -592,32 +623,107 @@ gint prepare_finish_page( GnomeDruidPage *page, GnomeDruid *druid )
 	gtk_clist_get_text( clist, row, 2, &fname );
 	gtk_clist_get_text( clist, row, 3, &lname );
 
-	gtk_object_set_data( GTK_OBJECT( druidpagefinish1 ), "uin", GINT_TO_POINTER( uin ) );
-	gtk_object_set_data( GTK_OBJECT( druidpagefinish1 ), "nick", g_strdup( nick ) );
-	
-	str = g_strdup_printf( _("Click \"Finish\" to add %d: %s (%s %s)\nto your Contact List."),
-	                       uin, nick, fname, lname );
+        Add_User( uin, nick, TRUE );
 
-	gnome_druid_page_finish_set_text (GNOME_DRUID_PAGE_FINISH (druidpagefinish1),
-	                                  str );
+        msg = g_strdup_printf(_("The UIN %d has been added.\nDo you want to add another one?"), uin);
+        finish_message = gnome_message_box_new (
+            msg,
+            GNOME_MESSAGE_BOX_QUESTION,
+            GNOME_STOCK_BUTTON_YES,
+            GNOME_STOCK_BUTTON_NO,
+            NULL);
+        switch(gnome_dialog_run(GNOME_DIALOG(finish_message))) 
+        {
+        case 0:
+            break;
+        case 1:
+            gtk_widget_destroy( GTK_WIDGET( window ) );
+            break;
+        }
 
-	g_free( str );
-
-	return FALSE;
+	return TRUE;
 }
 
-void finish_adding_user( GnomeDruidPage *page, GnomeDruid *druid )
+void sort_found_list( GtkCList *clist, gint column, gpointer data )
 {
-	UIN_T uin = GPOINTER_TO_INT( gtk_object_get_data( GTK_OBJECT( page ), "uin" ) );
-	gchar *nick = (gchar*)gtk_object_get_data( GTK_OBJECT( page ), "nick" );
+	gtk_clist_set_sort_column( clist, column );
 
-	if( nick == NULL )
-		nick = g_strdup_printf( "%d", uin );
-	else if( !strcmp( nick, "" ) )
+	/* Toggle the type of sort being performed. */
+	gtk_clist_set_sort_type( clist,
+	  ( clist->sort_type == GTK_SORT_ASCENDING ? GTK_SORT_DESCENDING :
+	    GTK_SORT_ASCENDING ) );
+
+	gtk_clist_freeze( clist );
+	gtk_clist_sort( clist );
+	gtk_clist_thaw( clist );
+}
+
+gint found_list_compare_func( GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2 )
+{
+	GtkCListRow *row1, *row2; /* Rows to compare. */
+	gint column; /* Column being sorted on */
+
+	g_return_val_if_fail( clist != NULL, 0 );
+	g_return_val_if_fail( GTK_IS_CLIST( clist ), 0 );
+
+	row1 = (GtkCListRow*)ptr1;
+	row2 = (GtkCListRow*)ptr2;
+	column = clist->sort_column;
+
+	g_return_val_if_fail( row1 != NULL && row2 != NULL, 0 );
+
+	if( row1->cell[column].type != GTK_CELL_TEXT ||
+	    row2->cell[column].type != GTK_CELL_TEXT )
 	{
-		g_free( nick );
-		nick = g_strdup_printf( "%d", uin );
+		int val1 = atoi( GTK_CELL_TEXT( row1->cell[column] )->text );
+		int val2 = atoi( GTK_CELL_TEXT( row2->cell[column] )->text );
+
+		return (val1 > val2 ? 1 : (val1 == val2 ? 0 : -1 ) );
 	}
 
-	Add_User( uin, nick, TRUE );
+	return 0;
+}
+
+void search_changed( GtkWidget *widget, gpointer data )
+{
+	GnomeDruid *druid;
+	GtkEntry *e_uin;
+	GtkEntry *e_nick;
+	GtkEntry *e_first;
+	GtkEntry *e_last;
+	GtkEntry *e_email;
+
+	druid = GNOME_DRUID( data );
+	e_uin = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_uin" ) );
+	e_nick = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_nick" ) );
+	e_first = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_first" ) );
+	e_last = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_last" ) );
+	e_email = GTK_ENTRY( gtk_object_get_data( GTK_OBJECT( druid ), "search_email" ) );
+
+	if( !strcmp( gtk_entry_get_text( e_uin ), "" ) &&
+	    !strcmp( gtk_entry_get_text( e_nick ), "" ) &&
+	    !strcmp( gtk_entry_get_text( e_first ), "" ) &&
+	    !strcmp( gtk_entry_get_text( e_last ), "" ) &&
+	    !strcmp( gtk_entry_get_text( e_email ), "" ) )
+		gnome_druid_set_buttons_sensitive( druid, FALSE, FALSE, TRUE );
+	else
+		gnome_druid_set_buttons_sensitive( druid, FALSE, TRUE, TRUE );
+}
+
+void clear_i_window2( GtkWidget *widget, gpointer data )
+{
+/*	UIN_T uin;
+
+	GSList *contact;
+
+	uin = GPOINTER_TO_INT( data );
+
+	contact = Find_User( uin );
+
+	if( contact != NULL )
+		kontakt->info->window = NULL;
+
+	if( New_Contact.uin == uin )*/
+		widget = NULL;
+		New_Contact.info->window = NULL;
 }
