@@ -1,6 +1,6 @@
 /*************************************
  Sends various messages to the server
- (c) 1999 Jeremy Wise 
+ (c) 1999 Jeremy Wise
  GnomeICU
 **************************************/
 
@@ -65,7 +65,7 @@ void Do_Resend( void )
 {
 	struct msg *queued_msg;
 	SIMPLE_MESSAGE_PTR s_mesg;
-	DWORD type; 
+	DWORD type;
 	char *data;
 	char *tmp;
 	char *temp;
@@ -89,7 +89,7 @@ void Do_Resend( void )
 			memcpy( temp, queued_msg->body, queued_msg->len);
 			SOCKWRITE_LOW( temp, queued_msg->len, "RESEND" );
 			g_free(temp);
-			queued_msg->exp_time = time(NULL) + 10; 
+			queued_msg->exp_time = time(NULL) + 10;
 			msg_queue_push( queued_msg );
 		}
 		else
@@ -101,7 +101,7 @@ void Do_Resend( void )
 				s_mesg = ( SIMPLE_MESSAGE_PTR ) pak.data;
 
 				type = Chars_2_Word(s_mesg->type);
-				data = s_mesg->len + 2; 
+				data = s_mesg->len + 2;
 				if ( type == URL_MESS || type == MRURL_MESS )
 				{
 					tmp = strchr( data, '\xFE' );
@@ -137,8 +137,8 @@ void Do_Resend( void )
 
 						while( contact != NULL )
 						{
-							((CONTACT_PTR)contact->data)->last_status = ((CONTACT_PTR)contact->data)->status;
-							((CONTACT_PTR)contact->data)->status = STATUS_OFFLINE;
+							kontakt->last_status = kontakt->status;
+							kontakt->status = STATUS_OFFLINE;
 							contact = contact->next;
 						}
 						Show_Quick_Status();
@@ -184,7 +184,7 @@ void Do_Resend( void )
 
 		if ( (queued_msg = msg_queue_peek() ) != NULL )
 		{
-			next_resend = queued_msg->exp_time; 
+			next_resend = queued_msg->exp_time;
 		}
 		else
 		{
@@ -205,7 +205,7 @@ void icq_sendmsg( UIN_T uin, const gchar *text, gboolean forced )
 {
 	SIMPLE_MESSAGE msg;
 	net_icq_pak pak;
-	int size, len; 
+	int size, len;
 	gchar *new_text;
 
 #ifdef TRACE_FUNCTION
@@ -225,6 +225,8 @@ void icq_sendmsg( UIN_T uin, const gchar *text, gboolean forced )
 
 	/* Add statement to personal history file */
 	add_outgoing_to_history( uin, new_text );
+	rus_conv(RUS_KOI_WIN, new_text);
+	kanji_conv_auto_s( &new_text, KANJI_SJIS );
 
 	if( !forced && !toggles->force && TCPSendMessage( uin, new_text) )
 	{
@@ -262,25 +264,42 @@ void icq_sendurl( GtkWidget *widget, struct URLInfo *data )
 {
 	SIMPLE_MESSAGE msg;
 	net_icq_pak pak;
-	gint size, len; 
+	gint size, len;
 	UIN_T uin;
 	gchar *new_text;
 	gchar *text;
+   	gchar *histtext;
 
 #ifdef TRACE_FUNCTION
 	g_print( "icq_sendurl\n" );
 #endif
 
 	uin = ((CONTACT_PTR)data->contact->data)->uin;
-	
+
+	if ( toggles->kanji ) {
+		gchar *ktmp =
+			kanji_conv_auto( gtk_entry_get_text( GTK_ENTRY( data->desc ) ),
+                             KANJI_SJIS);
+		text = g_strdup_printf( "%s\xFE%s", ktmp,
+								gtk_entry_get_text( GTK_ENTRY( data->url ) ) );
+		g_free( ktmp );
+	} else {
 	text = g_strdup_printf( "%s\xFE%s",
 				gtk_entry_get_text( GTK_ENTRY( data->desc ) ),
 				gtk_entry_get_text( GTK_ENTRY( data->url ) ) );
+	}
+   	histtext = g_strdup_printf( "URL: %s \nDescription: %s",
+				gtk_entry_get_text( GTK_ENTRY( data->url ) ),
+				gtk_entry_get_text( GTK_ENTRY( data->desc ) ) );
+
 	new_text = cr_convert( text );
+
 	g_free( text );
 
 	/* Add statement to personal history file */
-	add_outgoing_to_history( uin, new_text );
+	add_outgoing_to_history( uin, histtext );
+
+	g_free( histtext );
 
 	if( TCPSendURL( uin, new_text ) )
 	{
@@ -290,7 +309,7 @@ void icq_sendurl( GtkWidget *widget, struct URLInfo *data )
 	}
 
 	log_window_add( _("Sent URL (UDP)") , uin );
-	
+
 	len = strlen(new_text);
 	Word_2_Chars( pak.head.ver, ICQ_VER );
 	Word_2_Chars( pak.head.cmd, CMD_SENDM );
@@ -319,7 +338,7 @@ void icq_sendauthmsg( UIN_T uin)
 {
 	SIMPLE_MESSAGE msg;
 	net_icq_pak pak;
-	int size; 
+	int size;
 
 #ifdef TRACE_FUNCTION
 	g_print( "icq_sendauthmsg\n" );
@@ -332,7 +351,7 @@ void icq_sendauthmsg( UIN_T uin)
 
 	DW_2_Chars( msg.uin, uin );
 	DW_2_Chars( msg.type, AUTH_MESS );		/* A type authorization msg*/
-	Word_2_Chars( msg.len, 2 );		
+	Word_2_Chars( msg.len, 2 );
 
 	memcpy(&pak.data, &msg, sizeof( msg ) );
 
@@ -366,6 +385,18 @@ void icq_change_status( DWORD status )
 
 	SOCKWRITE( &(pak.head.ver), size + sizeof( pak.head ) - 2, "CMD_STATUS_CHANGE");
 
+	Word_2_Chars( pak.head.ver, ICQ_VER );
+	Word_2_Chars( pak.head.cmd, CMD_STATUS_CHANGE );
+	Word_2_Chars( pak.head.seq, seq_num++ );
+	DW_2_Chars( pak.head.UIN, our_info->uin );
+
+	DW_2_Chars( pak.data, ( status | (toggles->webpresence * 0x10000) ) );
+	Current_Status = status | (toggles->webpresence * 0x10000);
+
+	size = 4;
+
+	SOCKWRITE( &(pak.head.ver), size + sizeof( pak.head ) - 2, "CMD_STATUS_CHANGE");
+
 	ready_set();
 
 	if( toggles->applet )
@@ -383,16 +414,16 @@ void Quit_ICQ( void )
 #ifdef TRACE_FUNCTION
 	g_print( "Quit_ICQ\n" );
 #endif
-   
+
 	Word_2_Chars( pak.head.ver, ICQ_VER );
 	Word_2_Chars( pak.head.cmd, CMD_SEND_TEXT_CODE );
 	Word_2_Chars( pak.head.seq, seq_num++ );
 	DW_2_Chars( pak.head.UIN, our_info->uin );
-   
+
 	len = strlen( "B_USER_DISCONNECTED" ) + 1;
 	*(short * ) pak.data = len;
 	size = len + 4;
-   
+
 	memcpy( &pak.data[2], "B_USER_DISCONNECTED", len );
 	pak.data[ 2 + len ] = 05;
 	pak.data[ 3 + len ] = 00;
@@ -414,6 +445,7 @@ void send_info_req( UIN_T uin )
 #endif
 
 	info_req_99( uin );
+	/* FIXME: Why it returns explicitly? */
 	return;
 
 	Word_2_Chars( pak.head.ver, ICQ_VER );
@@ -429,8 +461,8 @@ void send_info_req( UIN_T uin )
 }
 
 /* Start a Server Search for Information Specified */
-void start_search( const gchar *email, const gchar *nick,
-                   const gchar* first, const gchar* last,
+void start_search( gchar *email, gchar *nick,
+                   gchar* first, gchar* last,
                    UIN_T uin )
 {
 	net_icq_pak pak;
@@ -447,6 +479,14 @@ void start_search( const gchar *email, const gchar *nick,
 		return;
 	}
 
+	rus_conv(RUS_KOI_WIN, nick);
+	rus_conv(RUS_KOI_WIN, first);
+	rus_conv(RUS_KOI_WIN, last);
+    if( toggles->kanji ) {
+        nick = kanji_conv_auto( nick, KANJI_SJIS );
+        first = kanji_conv_auto( first, KANJI_SJIS );
+        last = kanji_conv_auto( last, KANJI_SJIS );
+    }
 	Word_2_Chars( pak.head.ver, ICQ_VER );
 	Word_2_Chars( pak.head.cmd, CMD_SEARCH_USER );
 	Word_2_Chars( pak.head.seq, seq_num++ );
@@ -471,6 +511,11 @@ void start_search( const gchar *email, const gchar *nick,
 	size += strlen( email ) + 1;
 
 	SOCKWRITE( &(pak.head.ver), size + sizeof( pak.head ) - 2, "CMD_SEARCH_USER");
+    if( toggles->kanji) {
+        g_free(nick);
+        g_free(first);
+        g_free(last);
+    }
 }
 
 void send_ext_info_req( UIN_T uin )
@@ -493,7 +538,7 @@ void send_ext_info_req( UIN_T uin )
 	SOCKWRITE( &(pak.head.ver), size + sizeof( pak.head ) - 2, "CMD_EXT_INFO_REQ");
 }
 
-void reg_new_user( const gchar *pass )
+void reg_new_user( gchar *pass )
 {
 	net_icq_pak pak;
 	char len_buf[2];
@@ -502,6 +547,8 @@ void reg_new_user( const gchar *pass )
 #ifdef TRACE_FUNCTION
 	g_print( "reg_new_user\n" );
 #endif
+	rus_conv(RUS_KOI_WIN, pass); /* Bad, but need */
+	kanji_conv_auto_s( &pass, KANJI_SJIS ); /* Bad, but need? */
 
 	len = strlen( pass );
 	Word_2_Chars( pak.head.ver, ICQ_VER );
@@ -528,14 +575,14 @@ void reg_new_user( const gchar *pass )
 		DW_2_Chars( pak.head.session, our_session );
 
 	DW_2_Chars( pak.head.zero, 0L );
-	DW_2_Chars( pak.head.UIN, 0L ); 
-         
+	DW_2_Chars( pak.head.UIN, 0L );
+
 	SOCKWRITE_LOW( &( pak.head.ver ), size + sizeof( pak.head ) - 2, "CMD_REG_NEW_USER");
 }
 
 /***************************************************************
 This handles actually sending a packet after it's been assembled.
-When V5 is implemented this will wrinkle the packet and calculate 
+When V5 is implemented this will wrinkle the packet and calculate
 the checkcode.
 Adds packet to the queued messages.
 ****************************************************************/
@@ -544,10 +591,10 @@ size_t SOCKWRITE( void * ptr, size_t len, gchar *charid )
    struct msg *msg_to_queue;
 	static WORD seq2 = 0;
 	WORD temp;
-   WORD cmd;
+	WORD cmd;
 
 	DW_2_Chars( &((BYTE *)ptr)[2], 0L );
-	if ( 0 == our_session )
+	if ( our_session == 0 )
 	{
 		our_session = rand() & 0x3fffffff;
 	}
@@ -558,7 +605,7 @@ size_t SOCKWRITE( void * ptr, size_t len, gchar *charid )
 	if ( cmd != CMD_ACK && cmd != CMD_SEND_TEXT_CODE )
 	{
 		msg_to_queue = g_new0( struct msg, 1 );
-		if ( 0 == seq2 )
+		if ( seq2 == 0 )
 		{
 			seq2 = rand() & 0x7fff;
 		}
@@ -604,13 +651,14 @@ size_t SOCKREAD( void * ptr, size_t len )
 void Update_User_Info( void )
 {
 	net_icq_pak pak;
+	gchar *tmp;
 	int size ;
 
 	Word_2_Chars( pak.head.ver, ICQ_VER );
 	Word_2_Chars( pak.head.cmd, CMD_UPDATE_INFO );
 	Word_2_Chars( pak.head.seq, seq_num++ );
 	DW_2_Chars( pak.head.UIN, our_info->uin );
-   
+
    size = 0;
    Word_2_Chars( pak.data + size, strlen( our_info->nick ) + 1 );
    size += 2;
@@ -646,7 +694,7 @@ void Update_User_Info( void )
 	Word_2_Chars( pak.head.cmd, CMD_META_USER );
 	Word_2_Chars( pak.head.seq, seq_num++ );
 	DW_2_Chars( pak.head.UIN, our_info->uin );
-   
+
    Word_2_Chars( pak.data , META_INFO_SET );
    size = 2;
    Word_2_Chars( pak.data + size, strlen( our_info->nick ) + 1 );
@@ -710,12 +758,9 @@ void Update_User_Info( void )
 	Word_2_Chars( pak.head.cmd, CMD_META_USER );
 	Word_2_Chars( pak.head.seq, seq_num++ );
 	DW_2_Chars( pak.head.UIN, our_info->uin );
-   
+
    Word_2_Chars( pak.data , META_INFO_HOMEPAGE );
    size = 2;
-/*	pak.data[ size ++ ] = (BYTE)our_info->age;
-	Word_2_Chars( pak.data + size, our_info->sex );
-	Word_2_Chars( pak.data + size, 0x0200 );*/
 	Word_2_Chars( pak.data + size, our_info->age );
 	size += 2;
 	pak.data[ size++ ] = (BYTE)our_info->sex;
@@ -724,7 +769,7 @@ void Update_User_Info( void )
 	strcpy( pak.data + size, our_info->homepage );
 	size += strlen( our_info->homepage ) + 1;
 	pak.data[ size ++ ] = our_info->birth_year;
-	pak.data[ size ++ ] = our_info->birth_month;
+	pak.data[ size ++ ] = our_info->birth_month+1;
 	pak.data[ size ++ ] = our_info->birth_day;
 /* Three spoken languages */
 	pak.data[ size ++ ] = 0xff;
@@ -737,15 +782,20 @@ void Update_User_Info( void )
 	Word_2_Chars( pak.head.cmd, CMD_META_USER );
 	Word_2_Chars( pak.head.seq, seq_num++ );
 	DW_2_Chars( pak.head.UIN, our_info->uin );
-   
+
    Word_2_Chars( pak.data , META_INFO_ABOUT );
    size = 2;
-	Word_2_Chars( pak.data + size, strlen( cr_convert( our_info->about ) ) + 1 );
+	tmp = cr_convert( our_info->about );
+	Word_2_Chars( pak.data + size, strlen( tmp ) + 1 );
 	size += 2;
-	strcpy( pak.data + size, cr_convert( our_info->about ) );
-	size += strlen( cr_convert( our_info->about ) ) + 1;
+	strcpy( pak.data + size, tmp );
+	size += strlen( tmp ) + 1;
+	g_free( tmp );
 
 	SOCKWRITE( &(pak.head.ver), size + sizeof( pak.head ) - 2, "CMD_META_USER(META_INFO_ABOUT)");
+
+	/* This should send webpresence and auth information */
+	icq_sendwebpresence();
 }
 
 void info_req_99( UIN_T uin )
@@ -755,14 +805,7 @@ void info_req_99( UIN_T uin )
 	net_icq_pak pak;
 	int size;
 
-	contact = Contacts;
-
-	while( contact != NULL )
-	{
-		if( ((CONTACT_PTR)contact->data)->uin == uin )
-			break;
-		contact = contact->next;
-	}
+	contact = Find_User( uin );
 
 	if( contact == NULL )
 	{
@@ -770,7 +813,7 @@ void info_req_99( UIN_T uin )
 		our_contact->uin = uin;
 	}
 	else
-		our_contact = (CONTACT_PTR)contact->data;
+		our_contact = kontakt;
 
 	our_contact->info_seq = seq_num;
 
@@ -790,14 +833,14 @@ gchar *cr_convert( const gchar *text )
 {
 	char *t;
 	int cx, cy;
-	
+
 	cy = 0;
 	for( cx = 0; cx < strlen( text ); cx ++ )
 	{
 		if( text[ cx ] == '\n' )
 			cy ++;
 	}
-	
+
 	t = g_malloc( strlen( text ) + cy + 1 );
 
 	cy = 0;
@@ -813,7 +856,7 @@ gchar *cr_convert( const gchar *text )
 	}
 
 	t[ cy ] = 0x00;
-	
+
 	return t;
 }
 
@@ -845,7 +888,7 @@ void Gen_Checksum( BYTE * buf, DWORD len )
 {
    DWORD cc, cc2;
    DWORD r1,r2;
-   
+
    cc = buf[8];
    cc <<= 8;
    cc += buf[4];
@@ -857,36 +900,36 @@ void Gen_Checksum( BYTE * buf, DWORD len )
    r1 = rand() % ( len - 0x18 );
    r1 += 0x18;
    r2 = rand() & 0xff;
-   
+
    cc2 = r1;
    cc2 <<= 8;
    cc2 += buf[ r1 ];
    cc2 <<= 8;
-   cc2 += r2;   
+   cc2 += r2;
    cc2 <<= 8;
    cc2 += table[ r2 ];
    cc2 ^= 0xff00ff;
 /*   printf( "tbl[%02lX] = %02X\n%08lX\n\n", r2,table[ r2 ], cc ^ cc2 ); */
-   
+
    DW_2_Chars( &buf[0x14], cc ^ cc2 );
 }
 
 DWORD Scramble_cc( DWORD cc )
 {
     DWORD a[6];
-    
+
     a[1] = cc & 0x0000001F;
     a[2] = cc & 0x03E003E0;
     a[3] = cc & 0xF8000400;
     a[4] = cc & 0x0000F800;
     a[5] = cc & 0x041F0000;
-    
+
     a[1] <<= 0x0C;
     a[2] <<= 0x01;
     a[3] >>= 0x0A;
     a[4] <<= 0x10;
     a[5] >>= 0x0F;
-    
+
     return a[1] + a[2] + a[3] + a[4] + a[5];
 }
 
@@ -918,5 +961,5 @@ static size_t SOCKWRITE_LOW( void * ptr, size_t len, gchar *charid )
 	if( retval == -1 && Done_Login )
 		icq_set_status_offline( NULL );
 */
-	return retval; 
+	return retval;
 }
