@@ -17,10 +17,10 @@ void icq_sendauthmsg( UIN_T uin );
 void icq_change_status( DWORD status );
 void Quit_ICQ( void );
 void send_info_req( UIN_T uin );
-void start_search( const gchar *email, const gchar *nick,
-		   const gchar* first, const gchar* last, UIN_T uin );
+void start_search( gchar *email, gchar *nick,
+		   gchar* first, gchar* last, UIN_T uin );
 void send_ext_info_req( UIN_T uin );
-void reg_new_user( const gchar *pass );
+void reg_new_user( gchar *pass );
 size_t SOCKWRITE( void * ptr, size_t len, gchar *charid );
 size_t SOCKREAD( void * ptr, size_t len );
 void Update_User_Info( void );
